//+------------------------------------------------------------------+ 
//|                                        4_Level_ZZ_Semafor.mq4    | 
//|           based on original:           3_Level_ZZ_Semafor.mq4    | 
//+------------------------------------------------------------------+ 
#property copyright "asystem2000" 
#property link      "asystem2000@yandex.ru" 

/* History

2013-12-02 Modified by pips4life.  
   
   Renamed to 4_Level_ZZ_Semafor.mq4 because one new (higher degree) buffer set was added.  Fyi, to
   disable any degree, set its period to "0".
   
   Fixed the "delta" offset code so it properly provides a gap between the pivot and icon, but now it
   maintains the proper calculations for where the icons should occur.  User can now specify a value
   for "deltaInPips" (default=10.0).
   
   This version does NOT have any sound/email/popup alerts.  I don't know if any versions have acceptable
   alerts, given that this indicator is a "re-painting" type of indicator.  The bubbles that occur may well
   move and/or disappear as price moves higher/lower during the current bar and subsequent (future) bars.
   
   Most previous changes by TRO are part of other versions.  The starting point I used was the original, thus
   the TRO changes are NOT part of this one.
   
   Possible minor bug??  Search below for "What??(1)".  There is some useless code that sets "ExtMapBufferDn[shift]=res;"
   but that buffer already contains the same value "res".  This is as it was in the original version, however, maybe
   it's a bug and is supposed to be something else.  I have no idea.
   
2008-2013 by various.  There were several modifications, but some of these had/have bugs.
   3_Level_ZZ_Semafor1.mq4 was wrong in that it set every Dev_Step_# to "0,0" and ignored what was specified.
  
   The versions with "extern delta" was supposed to just offset the bubbles slightly above/below
   the pivots, but ended up screwing up the calculations for where the bubbles should go.

2008-02-26 http://codebase.mql4.com/2696

2008-01-23 by asystem2000.  Posted in a new thread here (Thank you!):
   http://www.forex-tsd.com/metatrader-4/12015-3_level_zz_semafor-indicator-mt4.html



*/

#property indicator_chart_window 
#property indicator_buffers 8
#property indicator_color1 Chocolate 
#property indicator_color2 Chocolate 
#property indicator_color3 MediumVioletRed
#property indicator_color4 MediumVioletRed
#property indicator_color5 Yellow
#property indicator_color6 Yellow
#property indicator_color7 Red
#property indicator_color8 Red

//---- input parameters 
extern double deltaInPips  = 10.0;
extern double Period1=5; 
extern double Period2=13; 
extern double Period3=34; 
extern double Period4=55; 
extern string   Dev_Step_1="1,3"; //Note it's "1,3" as was the original release.  It's not "3,1" as one might expect, but use whatever works...
extern string   Dev_Step_2="8,5";
extern string   Dev_Step_3="13,8"; // Fyi, this is "21,12" on MQL4 CODEBASE site.
extern string   Dev_Step_4="21,13";
extern int Symbol_1_Kod=140;
extern int Symbol_2_Kod=141;
extern int Symbol_3_Kod=142;
extern int Symbol_4_Kod=143;
extern int Symbol_1_Size=1 ;
extern int Symbol_2_Size=1;
extern int Symbol_3_Size=2;
extern int Symbol_4_Size=3;


//---- buffers 
double FP_BuferUp[];
double FP_BuferDn[]; 
double NP_BuferUp[];
double NP_BuferDn[]; 
double HP_BuferUp[];
double HP_BuferDn[]; 
double VP_BuferUp[];
double VP_BuferDn[]; 

int F_Period;
int N_Period;
int H_Period;
int V_Period;
int Dev1;
int Stp1;
int Dev2;
int Stp2;
int Dev3;
int Stp3;
int Dev4;
int Stp4;

double delta;
//+------------------------------------------------------------------+ 
//| Custom indicator initialization function                         | 
//+------------------------------------------------------------------+ 
int init() 
  { 
   double myPoint = getPoint(true);
   delta = deltaInPips*myPoint;
   
// ---------     
   if (Period1>0) F_Period=MathCeil(Period1*Period()); else F_Period=0; 
   if (Period2>0) N_Period=MathCeil(Period2*Period()); else N_Period=0; 
   if (Period3>0) H_Period=MathCeil(Period3*Period()); else H_Period=0; 
   if (Period4>0) V_Period=MathCeil(Period4*Period()); else V_Period=0; 
   
   IndicatorDigits(Digits); 
   
//----  1  
   if (Period1>0)
   {
   SetIndexStyle(0,DRAW_ARROW,0,Symbol_1_Size); 
   SetIndexArrow(0,Symbol_1_Kod); 
   SetIndexBuffer(0,FP_BuferUp); 
   SetIndexEmptyValue(0,0.0); 
   
   SetIndexStyle(1,DRAW_ARROW,0,Symbol_1_Size); 
   SetIndexArrow(1,Symbol_1_Kod); 
   SetIndexBuffer(1,FP_BuferDn); 
   SetIndexEmptyValue(1,0.0); 
   }
   
//----  2  
   if (Period2>0)
   {
   SetIndexStyle(2,DRAW_ARROW,0,Symbol_2_Size); 
   SetIndexArrow(2,Symbol_2_Kod); 
   SetIndexBuffer(2,NP_BuferUp); 
   SetIndexEmptyValue(2,0.0); 
   
   SetIndexStyle(3,DRAW_ARROW,0,Symbol_2_Size); 
   SetIndexArrow(3,Symbol_2_Kod); 
   SetIndexBuffer(3,NP_BuferDn); 
   SetIndexEmptyValue(3,0.0); 
   }
//----  3  
   if (Period3>0)
   {
   SetIndexStyle(4,DRAW_ARROW,0,Symbol_3_Size); 
   SetIndexArrow(4,Symbol_3_Kod); 
   SetIndexBuffer(4,HP_BuferUp); 
   SetIndexEmptyValue(4,0.0); 

   SetIndexStyle(5,DRAW_ARROW,0,Symbol_3_Size); 
   SetIndexArrow(5,Symbol_3_Kod); 
   SetIndexBuffer(5,HP_BuferDn); 
   SetIndexEmptyValue(5,0.0); 
   }
//
   if (Period4>0)
   {
   SetIndexStyle(6,DRAW_ARROW,0,Symbol_4_Size); 
   SetIndexArrow(6,Symbol_4_Kod); 
   SetIndexBuffer(6,VP_BuferUp); 
   SetIndexEmptyValue(6,0.0); 

   SetIndexStyle(7,DRAW_ARROW,0,Symbol_4_Size); 
   SetIndexArrow(7,Symbol_4_Kod); 
   SetIndexBuffer(7,VP_BuferDn); 
   SetIndexEmptyValue(7,0.0); 
   }
//     
   int CDev=0;
   int CSt=0;
   int Mass[]; 
   int C=0;  
   if (IntFromStr(Dev_Step_1,C, Mass)==1) 
      {
        Stp1=Mass[1];
        Dev1=Mass[0];
      }
   
   if (IntFromStr(Dev_Step_2,C, Mass)==1)
      {
        Stp2=Mass[1];
        Dev2=Mass[0];
      }      
   
   
   if (IntFromStr(Dev_Step_3,C, Mass)==1)
      {
        Stp3=Mass[1];
        Dev3=Mass[0];
      }      

   if (IntFromStr(Dev_Step_4,C, Mass)==1)
      {
        Stp4=Mass[1];
        Dev4=Mass[0];
      }      
   return(0); 
  } 
//+------------------------------------------------------------------+ 
//| Custor indicator deinitialization function                       | 
//+------------------------------------------------------------------+ 
int deinit() 
  { 
//---- 
    
//---- 
   return(0); 
  } 

//+------------------------------------------------------------------+ 
//| Custom indicator iteration function                              | 
//+------------------------------------------------------------------+ 
int start() 
  { 
   if (Period1>0) CountZZ(FP_BuferUp,FP_BuferDn,Period1,Dev1,Stp1);
   if (Period2>0) CountZZ(NP_BuferUp,NP_BuferDn,Period2,Dev2,Stp2);
   if (Period3>0) CountZZ(HP_BuferUp,HP_BuferDn,Period3,Dev3,Stp3);
   if (Period4>0) CountZZ(VP_BuferUp,VP_BuferDn,Period4,Dev4,Stp4);
      
   return(0); 
  } 
//+------------------------------------------------------------------+ 
//  
//int Take



//+------------------------------------------------------------------+ 
//|                           | 
//+------------------------------------------------------------------+  
int CountZZ( double& ExtMapBufferUp[], double& ExtMapBufferDn[], int ExtDepth, int ExtDeviation, int ExtBackstep )
  {
   int    shift, back,lasthighpos,lastlowpos;
   double val,res;
   double curlow,curhigh,lasthigh,lastlow;

   for(shift=Bars-ExtDepth; shift>=0; shift--)
     {
      val=Low[Lowest(NULL,0,MODE_LOW,ExtDepth,shift)];
      if(val==lastlow) val=0.0;
      else 
        { 
         lastlow=val; 
         if((Low[shift]-val)>(ExtDeviation*Point)) val=0.0;
         else
           {
            for(back=1; back<=ExtBackstep; back++)
              {
               res=ExtMapBufferUp[shift+back];
               if (res!=0.0) res = res+delta;
               if((res!=0.0)&&(res>val)) ExtMapBufferUp[shift+back]=0.0; 
              }
           }
        } 
        
          if (val==0.0) ExtMapBufferUp[shift] = 0.0;
          else ExtMapBufferUp[shift]=val-delta;
          
      //--- high
      val=High[Highest(NULL,0,MODE_HIGH,ExtDepth,shift)];
      if(val==lasthigh) val=0.0;
      else 
        {
         lasthigh=val;
         if((val-High[shift])>(ExtDeviation*Point)) val=0.0;
         else
           {
            for(back=1; back<=ExtBackstep; back++)
              {
               res=ExtMapBufferDn[shift+back];
               if(res!=0.0) res = res-delta;
               if((res!=0.0)&&(res<val)) ExtMapBufferDn[shift+back]=0.0; 
              } 
           }
        }
      
      if (val==0.0) ExtMapBufferDn[shift] = 0.0;
      else ExtMapBufferDn[shift]=val+delta;
     }
   // final cutting 
   lasthigh=-1; lasthighpos=-1;
   lastlow=-1;  lastlowpos=-1;

   for(shift=Bars-ExtDepth; shift>=0; shift--)
     {
      curlow=ExtMapBufferUp[shift];
      if (curlow != 0.0) curlow = curlow+delta;
      curhigh=ExtMapBufferDn[shift];
      if (curhigh != 0.0) curhigh = curhigh-delta;
      if((curlow==0.0)&&(curhigh==0.0)) continue;
      //---
      if(curhigh!=0.0)
        {
         if(lasthigh>0.0) 
           {
            if(lasthigh<curhigh) ExtMapBufferDn[lasthighpos]=0.0;
            else ExtMapBufferDn[shift]=0.0;
           }
         //---
         if(lasthigh<curhigh || lasthigh<0.0)
           {
            lasthigh=curhigh;
            lasthighpos=shift;
           }
         lastlow=-1;
        }
      //----
      if(curlow!=0.0)
        {
         if(lastlow>0.0)
           {
            if(lastlow>curlow) ExtMapBufferUp[lastlowpos]=0.0;
            else ExtMapBufferUp[shift]=0.0;
           }
         //---
         if((curlow<lastlow)||(lastlow<0.0))
           {
            lastlow=curlow;
            lastlowpos=shift;
           } 
         lasthigh=-1;
        }
     }
  
   for(shift=Bars-1; shift>=0; shift--)
     {
      if(shift>=Bars-ExtDepth) ExtMapBufferUp[shift]=0.0;
      else
        {
         // What??(1)  The next two lines do absolutely nothing because it just writes the value from the buffer back into the same buffer. Why is it here?? Bug??
         res=ExtMapBufferDn[shift];
         if(res!=0.0) ExtMapBufferDn[shift]=res;
        }
     }
 }
  
int Str2Massive(string VStr, int& M_Count, int& VMass[])
  {
    int val=StrToInteger( VStr);
    if (val>0)
       {
         M_Count++;
         int mc=ArrayResize(VMass,M_Count);
         if (mc==0)return(-1);
          VMass[M_Count-1]=val;
         return(1);
       }
    else return(0);    
  } 
  
  
int IntFromStr(string ValStr,int& M_Count, int& VMass[])
  {
    
    if (StringLen(ValStr)==0) return(-1);
    string SS=ValStr;
    int NP=0; 
    string CS;
    M_Count=0;
    ArrayResize(VMass,M_Count);
    while (StringLen(SS)>0)
      {
            NP=StringFind(SS,",");
            if (NP>0)
               {
                 CS=StringSubstr(SS,0,NP);
                 SS=StringSubstr(SS,NP+1,StringLen(SS));  
               }
               else
               {
                 if (StringLen(SS)>0)
                    {
                      CS=SS;
                      SS="";
                    }
               }
            if (Str2Massive(CS,M_Count,VMass)==0) 
               {
                 return(-2);
               }
      }
    return(1);    
  }
  
double getPoint(bool custommode)
{
   string symbol = Symbol();
   //double point = Point; // BUG: MT4 Build 392 did NOT update Point/Digits on a NEW CHART until AFTER the init() was done, so avoid and use MarketInfo
   double point = MarketInfo(symbol,MODE_POINT);
   int pluspos = StringFind(symbol,"+",0);
   int minuspos = StringFind(symbol,"-",0);
   if (pluspos > 0) symbol = StringSubstr(symbol,0,pluspos);
   else if (minuspos > 0) symbol = StringSubstr(symbol,0,minuspos);
   if (point < 0.000000001) point = MarketInfo(symbol,MODE_POINT); // try it again without timeshift
   
   if (! custommode) return(point);
   else
   {
      if (symbol == "NOKJPY" || symbol == "SEKJPY" || symbol == "GBPDKK" 
          || symbol == "GBPNOK" || symbol == "USDSKK" || symbol == "XAG") point = MarketInfo(symbol,MODE_POINT); //Point; // These are 0.001 on BroCo.
      else if (StringFind(symbol,"JPY",3) == 3 || symbol == "XAUUSD") point = 0.01; // ***JPY, XAUUSD
      else if (StringFind(symbol,"USD",0) >= 0
               || StringFind(symbol,"EUR",0) >= 0
               || StringFind(symbol,"GBP",0) >= 0
               || StringFind(symbol,"CAD",0) >= 0
               || StringFind(symbol,"NZD",0) >= 0
              ) point = 0.0001;
   }
   if (point < 0.000000001) point = Point;
   if (point < 0.000000001) point = 0.01; // A DEFAULT! Not necessarily a good choice but prevents divide-by-zero problems!
   //Print("getPoint: ",point,"  symbol: ",symbol);
   return(point);
} // end of getPoint

